from __future__ import division

from flat import rgb, shape, font, strike, paragraph, document
from flat.text import outlines

def plot(data, labelx, sublabelx, labely, sublabely, labelz, sublabelz):
    
    width, height = 110, 110 # size of the diagram in millimetres
    magnitude = 1.0 # upper bound of values to fit into the volume
    depth = 2.0 ** 0.5 * 0.5 # fake perspective proportion
    antidiagonal = False # whether the y axis runs from bottom left to upper right
    left, top, right, bottom = 10, 10, 40, 20 # padding around the diagram
    distance = 5.0 # gap for the labels
    
    sublabelz %= magnitude
    
    black = rgb(0, 0, 0)
    regular = font.open('Montserrat-Regular.ttf')
    bold = font.open('Montserrat-Bold.ttf')
    thin = shape().stroke(black).width(0.25)
    thick = shape().stroke(black).width(1.25)
    label = strike(regular).size(10, 12).color(black)
    labelbold = strike(bold).size(10, 12).color(black)
    d = document(width + left + right, height + top + bottom)
    p = d.addpage()
    
    m, n = len(data), len(data[0])
    w, h = n - 1.0, m - 1.0
    k = width / (w + depth * h)
    dx, dy = 0, height - h * k
    norm = (height - h*depth*k) / (magnitude*k)
    magnitude *= norm
    dx += left
    dy += top
    
    assert norm > 0, 'Diagram height too small.'
    
    if antidiagonal:
        points = [[((x+y*depth)*k, (h-y*depth-data[y][x]*norm)*k)
            for x in range(n)] for y in range(m)]
    else:
        points = [[((x+(h-y)*depth)*k, (h-y*depth-data[y][x]*norm)*k)
            for x in range(n)] for y in range(m)]
    
    for y in range(1, m - 1):
        coordinates = []
        for x in range(n):
            coordinates.extend(points[y][x])
        p.place(thick.polyline(*coordinates)).position(dx, dy)
    for x in range(1, n - 1):
        coordinates = []
        for y in range(m):
            coordinates.extend(points[y][x])
        p.place(thick.polyline(*coordinates)).position(dx, dy)
    
    coordinates = []
    for x in range(0, n - 1):
        coordinates.extend(points[0][x])
    for y in range(0, m - 1):
        coordinates.extend(points[y][n - 1])
    for x in reversed(range(1, n)):
        coordinates.extend(points[m - 1][x])
    for y in reversed(range(1, m)):
        coordinates.extend(points[y][0])
    p.place(thick.polygon(*coordinates)).position(dx, dy)
    
    x0, y0 = 0, h*k
    x1, y1 = w*k, h*(1-depth)*k
    x2, y2 = (w+h*depth)*k, (h*(1-depth)-magnitude)*k
    x3, y3 = h*depth*k, (h-magnitude)*k
    if antidiagonal:
        p.place(thin.polygon(x0, y0, x1, y0, x2, y1, x2, y2, x3, y2, x0, y3)).position(dx, dy)
        p.place(thin.polyline(x0, y3, x1, y3, x1, y0)).position(dx, dy)
        p.place(thin.line(x1, y3, x2, y2)).position(dx, dy)
        p.place(thin.polyline(x3, y2, x3, y1, x2, y1)).position(dx, dy)
        p.place(thin.line(x0, y0, x3, y1)).position(dx, dy)
        ax, ay = (x0+x1)*0.5, y0
        bx, by = x2, (y1+y2)*0.5
        cx, cy = (x1+x2)*0.5, (y0+y1)*0.5
        dist = distance - labelbold.leading*0.1 / p.k
    else:
        p.place(thin.polygon(x3, y0, x2, y0, x2, y3, x1, y2, x0, y2, x0, y1)).position(dx, dy)
        p.place(thin.polyline(x3, y0, x3, y3, x2, y3)).position(dx, dy)
        p.place(thin.line(x0, y2, x3, y3)).position(dx, dy)
        p.place(thin.polyline(x0, y1, x1, y1, x1, y2)).position(dx, dy)
        p.place(thin.line(x1, y1, x2, y0)).position(dx, dy)
        ax, ay = (x2+x3)*0.5, y0
        bx, by = x2, (y0+y3)*0.5
        cx, cy = (x1+x2)*0.5, (y2+y3)*0.5
        dist = -distance
    
    ex = labelbold.width(labelx)*0.5 / p.k
    fx, fy = labelbold.textsize*0.1 / p.k, labelbold.leading*0.5 / p.k
    gy = labelbold.ascender() / p.k
    p.place(outlines(
        labelbold.paragraph(labelx),
        label.paragraph(sublabelx))).position(ax+dx - ex, ay+distance+dy)
    p.place(outlines(
        labelbold.paragraph(labely),
        label.paragraph(sublabely))).position(cx+distance+dx, cy+dist+dy - gy)
    p.place(outlines(
        labelbold.paragraph(labelz),
        label.paragraph(sublabelz))).position(bx+distance+dx + fx, by+dy - fy)
    
    return d



